const app = getApp<IAppOption>();

// pages/story/a.ts
Component({
  /**
   * 组件的属性列表
   */
  properties: {
    slotIcon: {
      type: Boolean,
      value: false,
    },
    doubleConform: {
      type: Boolean,
      value: false,
    },
  },
  externalClasses: ["custom-record"],

  /**
   * 组件的初始数据
   */
  data: {
    record: false,
    startShow: false,
    show: false,
    audioObj: {} as any,

    recorderManager: null as null | WechatMiniprogram.RecorderManager,
    time: 0,
  },

  lifetimes: {
    attached() {
      const recorderManager = wx.getRecorderManager();
      this.setData({
        recorderManager,
      });
      this.recorderManagerListen(recorderManager);
    },
  },
  /**
   * 组件的方法列表
   */
  methods: {
    handleStart() {
      this.checkRecordPermission(() => {
        this.setData({
          startShow: true,
        });
      });
    },
    handleShow() {
      const { recorderManager } = this.data;
      this.setData({
        show: true,
        record: true,
      });
      if (this.data.startShow) {
        recorderManager?.start({
          format: "mp3",
          sampleRate: 16000,
        });
      }
    },
    handleHide() {
      const { recorderManager, startShow, doubleConform } = this.data;
      if (startShow) {
        recorderManager?.stop();
      }
      if (!doubleConform) {
        this.setData({
          startShow: false,
          show: false,
        });
      }
    },
    recorderManagerListen(recorderManager: WechatMiniprogram.RecorderManager) {
      const { doubleConform } = this.data;
      recorderManager.onStart(() => {
        this.setData({
          time: 0,
        });
        this.handleTiem();
      });
      recorderManager.onStop((res) => {
        this.handleHide();
        if (res.duration < 1000) {
          wx.showToast({
            title: "录音时间太短",
            icon: "none",
          });
          this.setData({
            startShow: false,
            show: false,
          });
          return;
        }
        this.setData({
          audioObj: {
            ...res,
            duration: Number.parseInt(String(res.duration / 1000)),
          },
        });
        if (doubleConform) {
          this.setData({
            record: false,
          });
        } else {
          this.handleConform();
        }
      });
    },
    handleCancel() {
      this.setData({
        startShow: false,
        show: false,
        record: false,
      });
    },
    handleConform() {
      const { audioObj } = this.data;
      this.handleCancel();
      this.uploadFile({
        fileType: "audio",
        tempFilePath: audioObj.tempFilePath,
        size: audioObj.fileSize,
        duration: audioObj.duration,
      }).then((res) => {
        this.triggerEvent("file", res as object);
      });
    },
    checkRecordPermission(callback: () => void) {
      wx.getSetting({
        success(res) {
          if (!res.authSetting["scope.record"]) {
            wx.authorize({
              scope: "scope.record",
              success() {
                // 用户已同意授权
                callback();
              },
              fail() {
                // 用户拒绝了授权
                wx.showModal({
                  confirmColor: "#00B4C5",
                  cancelColor: "#141515",
                  title: "提示",
                  content: "需要获取录音权限，请点击确定进入设置页面进行授权。",
                  showCancel: false,
                  success(res) {
                    if (res.confirm) {
                      wx.openSetting({
                        success(res) {
                          // 用户在设置页面设置了录音权限
                          if (res.authSetting["scope.record"]) {
                            // 用户在设置页面设置了录音权限
                            callback();
                          } else {
                            // 用户在设置页面未设置录音权限
                          }
                        },
                      });
                    }
                  },
                });
              },
            });
          } else {
            // 已经有录音权限
            callback();
          }
        },
      });
    },
    handleTiem() {
      const time = setTimeout(() => {
        clearInterval(time);
        if (!this.data.show) return;
        this.setData({
          time: this.data.time + 1,
        });
        this.handleTiem();
      }, 1000);
    },
    uploadFile(item) {
      wx.showLoading({
        title: "正在上传",
      });
      return new Promise((resolve, reject) => {
        let url = `${app.globalData.upFileUrl}?r=file-service/upload-`;
        if (item.fileType === "image") {
          url += "img";
        } else if (item.fileType === "video") {
          url += "video";
        } else if (item.fileType === "audio") {
          url += "audio";
        } else {
          url += "doc";
        }
        wx.uploadFile({
          filePath: item.tempFilePath,
          name: "file",
          url,
          success: (res) => {
            wx.hideLoading();
            const data = JSON.parse(res.data);

            const suffix = this.GetExtensionFileName(data.data.Url);
            const expandJson = {
              fileId: "",
              name: `病历相关文件.${suffix}`,
              size: (item.size / 1024).toFixed(2),
              fileUrl: data.data.Url,
              suffix,
              type: item.fileType,
              imgUrl: "",
              duration: 0,
            };
            if (item.fileType === "image") {
              expandJson.imgUrl = data.data.Url;
            }
            if (item.fileType === "video") {
              expandJson.imgUrl = data.data.SnapshotUrl;
            }
            if (item.fileType === "audio") {
              expandJson.duration = item.duration;
            }
            resolve(expandJson);
          },
          fail() {
            wx.hideLoading();
            reject(new Error("上传失败"));
          },
        });
      });
    },
    GetExtensionFileName(pathfilename) {
      const reg = /(\\+)/g;
      const pString = pathfilename.replace(reg, "#"); //用正则表达式来将\或\\替换成#
      const arr = pString.split("#"); // 以“#”为分隔符，将字符分解为数组 例如 D Program Files bg.png
      const lastString = arr[arr.length - 1]; //取最后一个字符
      const arr2 = lastString.split("."); //   再以"."作为分隔符
      return arr2[arr2.length - 1]; //将后缀名返回出来
    },
    noneEnoughPeople() {},

    handleCatchTap() {
      return false;
    },
  },
});

export {};
